#
# Find an Litmus installation or build tree.
#
# When Litmus is found, the LitmusConfig.cmake file is sourced to setup the
# location and configuration of Litmus.  Please read this file, or
# LitmusConfig.cmake.in from the Litmus source tree for the full list of
# definitions.  Of particular interest is
#
# Litmus_USE_FILE   - A CMake source file that can be included
#                     to set the include directories, library
#                     directories, and preprocessor macros.
#
# In addition to the variables read from LitmusConfig.cmake, this find
# module also defines
#
# Litmus_DIR        - The directory containing LitmusConfig.cmake.  This is either
#                     the root of the build tree, or the lib/InsightToolkit
#                     directory.  This is the only cache entry.
#
# Litmus_FOUND      - Whether Litmus was found.  If this is true, Litmus_DIR is okay.
#
# USE_Litmus_FILE   - The full path to the UseLitmus.cmake file.  This is provided
#                     for backward compatability.  Use Litmus_USE_FILE instead.
#

SET(Litmus_DIR_STRING "directory containing LitmusConfig.cmake.  This is either the root of the build tree, or PREFIX/lib/Litmus for an installation.")

# Search only if the location is not already known.
IF(NOT Litmus_DIR)
  # Get the system search path as a list.
  IF(UNIX)
    STRING(REGEX MATCHALL "[^:]+" Litmus_DIR_SEARCH1 "$ENV{PATH}")
  ELSE(UNIX)
    STRING(REGEX REPLACE "\\\\" "/" Litmus_DIR_SEARCH1 "$ENV{PATH}")
  ENDIF(UNIX)
  STRING(REGEX REPLACE "/;" ";" Litmus_DIR_SEARCH2 ${Litmus_DIR_SEARCH1})

  # Construct a set of paths relative to the system search path.
  SET(Litmus_DIR_SEARCH "")
  FOREACH(dir ${Litmus_DIR_SEARCH2})
    SET(Litmus_DIR_SEARCH ${ITK_DIR_SEARCH} "${dir}/../lib/Litmus")
  ENDFOREACH(dir)

  #
  # Look for an installation or build tree.
  #
  FIND_PATH(Litmus_DIR LitmusConfig.cmake
    # Look for an environment variable Litmus_DIR.
    $ENV{Litmus_DIR}

    # Look in places relative to the system executable search path.
    ${Litmus_DIR_SEARCH}

    # Look in standard UNIX install locations.
    /usr/local/lib/InsightToolkit
    /usr/lib/InsightToolkit

    # Read from the CMakeSetup registry entries.  It is likely that
    # Litmus will have been recently built.
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild1]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild2]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild3]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild4]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild5]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild6]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild7]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild8]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild9]
    [HKEY_CURRENT_USER\\Software\\Kitware\\CMakeSetup\\Settings\\StartPath;WhereBuild10]

    # Help the user find it if we cannot.
    DOC "The ${Litmus_DIR_STRING}"
  )
ENDIF(NOT Litmus_DIR)

# If Litmus was found, load the configuration file to get the rest of the
# settings.
IF(Litmus_DIR)
  SET(Litmus_FOUND 1)
  INCLUDE(${Litmus_DIR}/LitmusConfig.cmake)

  # Set USE_Litmus_FILE for backward-compatability.
  SET(USE_Litmus_FILE ${Litmus_USE_FILE})
ELSE(Litmus_DIR)
  SET(Litmus_FOUND 0)
  IF(Litmus_FIND_REQUIRED)
    MESSAGE(FATAL_ERROR "Please set Litmus_DIR to the ${Litmus_DIR_STRING}")
  ENDIF(Litmus_FIND_REQUIRED)
ENDIF(Litmus_DIR)
